@@ -115,26 +115,36 @@ rbart_vi <- function(
   if (is.null(node.prior)) samplerArgs[["node.prior"]] <- NULL
 
   chainResults <- vector("list", n.chains)
-  if (n.threads == 1L || n.chains == 1L) {
-    for (chainNum in seq_len(n.chains))
-      chainResults[[chainNum]] <- rbart_vi_fit(samplerArgs, group.by, prior)
-  } else {
-    cluster <- makeCluster(n.threads)
+  runSingleThreaded <- n.threads <= 1L || n.chains <= 1L
+  if (!runSingleThreaded) {
+    tryResult <- tryCatch(cluster <- makeCluster(min(n.threads, n.chains), "PSOCK"), error = function(e) e)
+    if (is(tryResult, "error"))
+      tryResult <- tryCatch(cluster <- makeCluster(min(n.threads, n.chains), "FORK"), error = function(e) e)
     
-    clusterExport(cluster, "rbart_vi_fit", asNamespace("dbarts"))
-    clusterEvalQ(cluster, require(dbarts))
+    if (is(tryResult, "error")) {
+      warning("unable to multithread, defaulting to single: ", tryResult$message)
+      runSingleThreaded <- TRUE
+    } else {
+      clusterExport(cluster, "rbart_vi_fit", asNamespace("dbarts"))
+      clusterEvalQ(cluster, require(dbarts))
+      
+      tryResult <- tryCatch(
+        chainResults <- clusterMap(cluster, "rbart_vi_fit", seq_len(n.chains), MoreArgs = namedList(samplerArgs, group.by, prior)))
     
-    tryResult <- tryCatch(
-      chainResults <- clusterCall(cluster, "rbart_vi_fit", samplerArgs, group.by, prior))
-    
-    stopCluster(cluster)
+      stopCluster(cluster)
+    }
+  }
+
+  if (runSingleThreaded) {
+    for (chainNum in seq_len(n.chains))
+      chainResults[[chainNum]] <- rbart_vi_fit(1L, samplerArgs, group.by, prior)
   }
-  
   packageRbartResults(control, data, group.by, group.by.test, chainResults, combineChains)
 }
 
-rbart_vi_fit <- function(samplerArgs, group.by, prior) 
+rbart_vi_fit <- function(chain.num, samplerArgs, group.by, prior) 
 {
+  chain.num <- "ignored"
   sampler <- do.call(dbarts::dbarts, samplerArgs)
   sampler$control@call <- samplerArgs$control@call
   
@@ -189,7 +199,7 @@ rbart_vi_fit <- function(samplerArgs, group.by, prior)
     treeFit.train <- sampler$predict(sampler$data@x) - ranef.vec
     
     # order of update matters - need to store a ranef that goes with a prediction
-    # or else when they're added together they won't be consistent with `predict'
+    # or else when they're added together they won't be consistent with `predict`
     for (i in seq_len(control@n.burn)) {
       # update ranef
       resid <- y.st - treeFit.train
