@@ -29,6 +29,7 @@ namespace dbarts {
   struct Control;
   struct Model;
   struct Data;
+  struct State;
   
   struct CGMPrior;
   struct NormalPrior;
@@ -45,8 +46,9 @@ extern "C" {
   dbarts::Control* dbarts_createControl(SEXP controlExpr);
   void dbarts_destroyControl(dbarts::Control* control);
   void dbarts_initializeControl(dbarts::Control* control, SEXP controlExpr);
-  // void dbarts_invalidateControl(dbarts::Control* control); // invalidation not necessary, owns no memory
-  
+  // void dbarts_invalidateControl(dbarts::Control* control); // invalidation not necessary, allocates no memory
+  void dbarts_setControl(dbarts::BARTFit* fit, const dbarts::Control* control);
+    
   dbarts::Data* dbarts_createData(SEXP dataExpr);
   void dbarts_destroyData(dbarts::Data* data);
   void dbarts_initializeData(dbarts::Data* data, SEXP dataExpr);
@@ -62,14 +64,24 @@ extern "C" {
   void dbarts_destroyFit(dbarts::BARTFit* fit);
   void dbarts_invalidateFit(dbarts::BARTFit* fit);
   
+  SEXP dbarts_createStateExpression(const dbarts::BARTFit* fit);
+  void dbarts_initializeState(dbarts::BARTFit* fit, SEXP stateExpr);
+  
   void dbarts_setRNGState(dbarts::BARTFit* fit, const void* const* uniformState, const void* const* normalState);
   
   void dbarts_printInitialSummary(const dbarts::BARTFit* fit);
+  void dbarts_printTrees(const dbarts::BARTFit* fit,
+                         const std::size_t* chains,  std::size_t numChains,
+                         const std::size_t* samples, std::size_t numSamples,
+                         const std::size_t* indices, std::size_t numIndices);
+  
   dbarts::Results* dbarts_runSampler(dbarts::BARTFit* fit);
   dbarts::Results* dbarts_runSamplerForIterations(dbarts::BARTFit* fit, std::size_t numBurnIn, std::size_t numSamples);
   void dbarts_runSamplerWithResults(dbarts::BARTFit* fit, std::size_t numBurnIn, dbarts::Results* results);
   void dbarts_sampleTreesFromPrior(dbarts::BARTFit* fit);
+  void dbarts_sampleNodeParametersFromPrior(dbarts::BARTFit* fit);
   
+  void dbarts_predict(const dbarts::BARTFit* fit, const double* x_test, std::size_t numTestObservations, const double* testOffset, double* result);
   // 'settors' simply replace local pointers to variables. dimensions much match
   // 'update' modifies the local copy (which may belong to someone else)
   void dbarts_setResponse(dbarts::BARTFit* fit, const double* newResponse);
