@@ -292,6 +292,7 @@ namespace {
     DEF_FUNC("dbarts_create", create, 3),
     DEF_FUNC("dbarts_run", run, 3),
     DEF_FUNC("dbarts_sampleTreesFromPrior", sampleTreesFromPrior, 1),
+    DEF_FUNC("dbarts_sampleNodeParametersFromPrior", sampleNodeParametersFromPrior, 1),
     DEF_FUNC("dbarts_printTrees", printTrees, 4),
     DEF_FUNC("dbarts_getTrees", getTrees, 4),
     DEF_FUNC("dbarts_predict", predict, 3),
@@ -367,6 +368,7 @@ namespace {
     DEF_FUNC("initializeControl", dbarts_initializeControl),
     DEF_FUNC("destroyControl", dbarts_destroyControl),
     // DEF_FUNC("invalidateControl", dbarts_invalidateControl),
+    DEF_FUNC("setControl", dbarts_setControl),
     
     DEF_FUNC("createData", dbarts_createData),
     DEF_FUNC("initializeData", dbarts_initializeData),
@@ -383,13 +385,19 @@ namespace {
     DEF_FUNC("destroyFit", dbarts_destroyFit),
     DEF_FUNC("invalidateFit", dbarts_invalidateFit),
     
+    DEF_FUNC("createStateExpression", dbarts_createStateExpression),
+    DEF_FUNC("initializeState", dbarts_initializeState),
+    
     DEF_FUNC("printInitialSummary", dbarts_printInitialSummary),
+    DEF_FUNC("printTrees", dbarts_printTrees),
     DEF_FUNC("setRNGState", dbarts_setRNGState),
     
     DEF_FUNC("runSampler", dbarts_runSampler),
     DEF_FUNC("runSamplerForIterations", dbarts_runSamplerForIterations),
     DEF_FUNC("runSamplerWithResults", dbarts_runSamplerWithResults),
     DEF_FUNC("sampleTreesFromPrior", dbarts_sampleTreesFromPrior),
+    DEF_FUNC("sampleNodeParametersFromPrior", dbarts_sampleNodeParametersFromPrior),
+    DEF_FUNC("predict", dbarts_predict),
     DEF_FUNC("setResponse", dbarts_setResponse),
     DEF_FUNC("setOffset", dbarts_setOffset),
     DEF_FUNC("setSigma", dbarts_setSigma),
