@@ -11,7 +11,7 @@
 namespace dbarts {
   using std::size_t;
   
-  double computeLogLikelihoodForBranch(const BARTFit& fit, const Node& branch, const double* y)
+  double computeLogLikelihoodForBranch(const BARTFit& fit, size_t chainNum, const Node& branch, const double* y, double sigma)
   {
     NodeVector bottomVector(branch.getBottomVector());
     size_t numBottomNodes = bottomVector.size();
@@ -22,7 +22,7 @@ namespace dbarts {
       
       if (bottomNode.getNumObservations() == 0) return -10000000.0;
       
-      logProbability += fit.model.muPrior->computeLogIntegratedLikelihood(fit, bottomNode, y, fit.state.sigma * fit.state.sigma);
+      logProbability += fit.model.muPrior->computeLogIntegratedLikelihood(fit, chainNum, bottomNode, y, sigma * sigma);
     }
     
     return logProbability;
